@@ -1,4 +1,4 @@
-package Dancer::Plugin::Catmandu::OAI; # TODO hierarchical sets, setDescription
+package Dancer::Plugin::Catmandu::OAI;
 
 =head1 NAME
 
@@ -6,10 +6,10 @@ Dancer::Plugin::Catmandu::OAI - OAI-PMH provider backed by a searchable Catmandu
 
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.0401';
 
 use Catmandu::Sane;
-use Catmandu::Util qw(:is);
+use Catmandu::Util qw(is_string is_array_ref);
 use Catmandu;
 use Catmandu::Fix;
 use Catmandu::Exporter::Template;
@@ -208,13 +208,24 @@ $template_record_header
 $template_footer
 TT
 
+    my $admin_email = $setting->{adminEmail} // [];
+    $admin_email = [$admin_email] unless is_array_ref($admin_email);
+    $admin_email = join('', map { "<adminEmail>$_</adminEmail>" } @$admin_email);
+
+    my @identify_extra_fields;
+    for my $i_field (qw(description compression)){
+        my $i_value = $setting->{$i_field} // [];
+        $i_value = [$i_value] unless is_array_ref($i_value);
+        push @identify_extra_fields, join('', map { "<$i_field>$_</$i_field>" } @$i_value);
+    }
+
     my $template_identify = <<TT;
 $template_header
 <Identify>
 <repositoryName>$setting->{repositoryName}</repositoryName>
 <baseURL>[% uri_base %]</baseURL>
 <protocolVersion>2.0</protocolVersion>
-<adminEmail>$setting->{adminEmail}</adminEmail>
+$admin_email
 <earliestDatestamp>$setting->{earliestDatestamp}</earliestDatestamp>
 <deletedRecord>$setting->{deletedRecord}</deletedRecord>
 <granularity>$setting->{granularity}</granularity>
@@ -228,6 +239,7 @@ $template_header
         <sampleIdentifier>$setting->{sampleIdentifier}</sampleIdentifier>
     </oai-identifier>
 </description>
+@identify_extra_fields
 $branding
 </Identify>
 $template_footer
@@ -298,6 +310,13 @@ TT
 <set>
     <setSpec>$set->{setSpec}</setSpec>
     <setName>$set->{setName}</setName>
+TT
+
+    my $set_descriptions = $set->{setDescription} // [];
+    $set_descriptions = [$set_descriptions] unless is_array_ref($set_descriptions);
+    $template_list_sets .= "<setDescription>$_</setDescription>" for @$set_descriptions;
+
+    $template_list_sets .= <<TT;
 </set>
 TT
     }
@@ -698,7 +717,9 @@ The Dancer configuration file 'config.yml' contains basic information for the OA
     * datestamp_field - Which field in the record contains a datestamp ('datestamp' in our example above)
     * repositoryName - The name of the repository
     * uri_base - The base URL of the repository
-    * adminEmail - An administrative email
+    * adminEmail - An administrative email. Can be string or array of strings. This will be included in the Identify response.
+    * compression - a compression encoding supported by the repository. Can be string or array of strings. This will be included in the Identify response.
+    * description - XML container that describes your repository. Can be string or array of strings. This will be included in the Identify response. Note that this module will try to validate the XML data.
     * earliestDatestamp - The earliest datestamp available in the dataset an YYYY-MM-DDTHH:MM:SSZ
     * deletedRecord - The policy for deleted records. See also: L<https://www.openarchives.org/OAI/openarchivesprotocol.html#DeletedRecords>
     * repositoryIdentifier - A prefix to use in OAI-PMH identifiers
@@ -715,6 +736,7 @@ The Dancer configuration file 'config.yml' contains basic information for the OA
     * sets - Optional an array of OAI-PMH sets and the CQL query to retrieve records in this set from the Catmandu::Store
         * setSpec - A short string for the same of the set
         * setName - A longer description of the set
+        * setDescription - an optional and repeatable container that may hold community-specific XML-encoded data about the set. Should be string or array of strings.
         * cql - The CQL command to find records in this set in the L<Catmandu::Store>
     * xsl_stylesheet - Optional path to an xsl stylesheet
 
